#include <cstdio>
#include <iostream>
#include <string>
#include <map>
#include <vector>

using namespace std;


void readFile(char* fileName, string& str)
{
	FILE* file = fopen(fileName, "r");
	if (file)
	{
		while (true)
		{
			char ch = fgetc(file);
			if (feof(file))
			{
				break;
			}
			str.push_back(ch);
		}
		fclose(file);
	}
}

typedef map<string, string> StringMap;

void readString(const string& str, string& httpQuery, StringMap& strMap)
{
	size_t pos = str.find_first_of('?');
	if (pos != string::npos)
	{
		httpQuery = str.substr(0, pos);
		size_t start = pos + 1;
		size_t end;
		while ((end = str.find_first_of('=', start)) != string::npos)
		{
			string key, value;
			key = str.substr(start, end - start);
			start = end + 1;
			if ((end = str.find_first_of('&', start)) != string::npos)
			{
				value = str.substr(start, end - start);
				start = end + 1;
				strMap.insert(make_pair(key, value));
			}
			else
			{
				// end of file
				value = str.substr(start);
				strMap.insert(make_pair(key, value));
				break;
			}
		}
	}
}

typedef vector<string> StringVector;

char* CountryList[] =
{
	"au.txt", "br.txt", "ca.txt", "cn.txt", "de.txt", "es.txt", "fr.txt", "hk.txt", "in.txt", "jp.txt",
			"kr.txt", "my.txt", "ru.txt", "se.txt", "tw.txt", "uk.txt", "us.txt"
};


void readAllFiles(StringVector& strVector)
{
	for (int i = 0; i < sizeof(CountryList)/sizeof(char*); i ++)
	{
		string str;
		char path[256];
		sprintf(path, "/home/nick/%s", CountryList[i]);
		readFile(path, str);
		strVector.push_back(str);
	}
}

void myCompare(const string& str1, const string& str2)
{
	StringMap strMap1, strMap2;
	string http1, http2;
	readString(str1, http1, strMap1);
	readString(str2, http2, strMap2);

	vector<pair<string, string> > equal;
	vector<pair<string, string> > diff, strMap1Only, strMap2Only;
	StringMap::const_iterator src, dst;
	for (src = strMap1.begin(); src!= strMap1.end(); src ++)
	{
		dst = strMap2.find(src->first);
		if (dst != strMap2.end())
		{
			if (src->second == dst->second)
			{
				equal.push_back(*src);
			}
			else
			{
				diff.push_back(*src);
				diff.push_back(*dst);
			}
		}
		else
		{
			strMap1Only.push_back(*src);
		}
	}

	for (src = strMap2.begin(); src != strMap2.end(); src ++)
	{
		dst = strMap1.find(src->first);
		// now we only care about those unique in strMap2
		if (dst == strMap1.end())
		{
			strMap2Only.push_back(*src);
		}
	}
	cout << "htt1=" << http1 << endl;
	cout << "htt2=" << http2 << endl;

	cout<<" the identical parts:" << endl;
	int i;
	for (i = 0; i < equal.size(); i ++)
	{
		cout << equal[i].first << " = " << equal[i].second <<endl;
	}
	cout << " the different parts: " << endl;
	for (i = 0; i < diff.size(); i ++)
	{
		cout << diff[i].first << " = " << diff[i].second << endl;
	}
	cout << " let's compare GL part:" << endl;

	string gl1, gl2;
	src = strMap1.find("SO_GL");
	dst = strMap2.find("SO_GL");
	if (src != strMap1.end() && dst != strMap2.end())
	{
		gl1 = src->second;
		gl2 = dst->second;
		for (i = 0; i < gl1.size(); i ++)
		{
			if (i < gl2.size())
			{
				if (gl1[i] == gl2[i])
				{
					gl1[i] = gl2[i] = '*';
				}
			}
		}
		cout << "GL 1:" << endl;
		cout << gl1 <<endl;
		cout << "GL 1:" << endl;
		cout << gl2 << endl;
	}
	if (strMap1Only.size())
	{
		cout << " strMap1 only:";
		for (i = 0; i < strMap1Only.size(); i ++)
		{
			cout << strMap1Only[i].first << " = " << strMap1Only[i].second <<endl;
		}
	}
	if (strMap2Only.size())
	{
		cout << " strMap2 only:";
		for (i = 0; i < strMap2Only.size(); i ++)
		{
			cout << strMap2Only[i].first << " = " << strMap2Only[i].second <<endl;
		}
	}
}

void analysis(const StringVector& strVector)
{
	for (int i = 0; i < strVector.size(); i ++)
	{
		const string& str1 = strVector[i];
		for (int j = 0; j < strVector.size(); j ++)
		{
			if (i != j)
			{
				const string& str2 = strVector[j];
				myCompare(str1, str2);
			}
		}
	}
}

int main(int argc, char** argv)
{
	StringVector strVector;
	readAllFiles(strVector);
	analysis(strVector);

	return 0;
}
